/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef MUTT_LIB_MIME_MIME_H
#define MUTT_LIB_MIME_MIME_H

#include <stdlib.h>

#include <lib-lib/mem.h>
#include <lib-lib/buffer.h>
#include <lib-lib/list.h>

/* Content-Type */
enum {
    TYPEOTHER,
    TYPEAUDIO,
    TYPEAPPLICATION,
    TYPEIMAGE,
    TYPEMESSAGE,
    TYPEMODEL,
    TYPEMULTIPART,
    TYPETEXT,
    TYPEVIDEO,
    TYPEANY
};

/* Content-Transfer-Encoding */
enum {
    ENCOTHER,
    ENC7BIT,
    ENC8BIT,
    ENCQUOTEDPRINTABLE,
    ENCBASE64,
    ENCBINARY,
    ENCUUENCODED
};

/* Content-Disposition values */
enum {
    DISPINLINE,
    DISPATTACH,
    DISPFORMDATA
};

typedef struct parameter {
    char *attribute;
    char *value;
    struct parameter *next;
} PARAMETER;

typedef struct address_t {
    char *personal;               /* real name of address */
    char *mailbox;                /* mailbox and host address */
    int group;                    /* group mailbox? */
    struct address_t *next;
} address_t;

typedef struct envelope_t {
    address_t *return_path;
    address_t *from;
    address_t *to;
    address_t *cc;
    address_t *bcc;
    address_t *sender;
    address_t *reply_to;
    address_t *mail_followup_to;
    char *list_post;              /* this stores a mailto URL, or nothing */
    char *subject;
    char *real_subj;              /* offset of the real subject */
    char *message_id;
    char *supersedes;
    char *date;
    char *x_label;
    char *organization;
#ifdef USE_NNTP
    char *newsgroups;
    char *xref;
    char *followup_to;
    char *x_comment_to;
#endif
    BUFFER *spam;
    LIST *references;             /* message references (in reverse order) */
    LIST *in_reply_to;            /* in-reply-to header content */
    LIST *userhdrs;               /* user defined headers */
    unsigned int irt_changed:1;   /* In-Reply-To changed to link/break threads */
    unsigned int refs_changed:1;  /* References changed to break thread */
} ENVELOPE;


DO_INIT(address_t, address);
void address_wipe(address_t *);

DO_NEW(address_t, address);
DO_DELETE(address_t, address);
DO_SLIST(address_t, address);


extern const char MimeSpecials[];
extern const char *BodyTypes[];
extern const char *BodyEncodings[];

/* MIME encoding/decoding global vars */

#define is_multipart(x) \
    ((x)->type == TYPEMULTIPART \
     || ((x)->type == TYPEMESSAGE && (!strcasecmp((x)->subtype, "rfc822") \
                                      || !strcasecmp((x)->subtype, "news"))))
#define TYPE(X) ((X->type == TYPEOTHER) && (X->xtype != NULL) ? X->xtype : BodyTypes[(X->type)])
#define ENCODING(X) BodyEncodings[(X)]

/****************************************************************************/
/* RFC 822                                                                  */
/*                Standard for ARPA Internet Text Messages                  */
/****************************************************************************/

address_t *address_dup(address_t *addr);
address_t *address_list_dup(address_t *addr);
void rfc822_qualify(address_t *, const char *);

address_t *rfc822_parse_adrlist(address_t *, const char *s);

ssize_t rfc822_write_address(char *, ssize_t, address_t *, int);
ssize_t rfc822_write_address_single(char *, ssize_t, address_t *, int);
ssize_t rfc822_strcpy(char *, ssize_t, const char *, const char *);

extern const char RFC822Specials[];

/****************************************************************************/
/* RFC 2047                                                                 */
/*       MIME (Multipurpose Internet Mail Extensions) Part Three:           */
/*             Message Header Extensions for Non-ASCII Text                 */
/****************************************************************************/

char *mutt_choose_charset(const char *fromcode, const char *charsets,
                          char *u, ssize_t ulen, char **d, ssize_t *dlen);

void rfc2047_encode_string(char **);
void rfc2047_encode_adrlist(address_t *, const char *);

void rfc2047_decode(char **);
void rfc2047_decode_adrlist(address_t *);
void rfc2047_decode_envelope(ENVELOPE* e);

/****************************************************************************/
/* RFC 2231                                                                 */
/*             MIME Parameter Value and Encoded Word Extensions:            */
/*               Character Sets, Languages, and Continuations               */
/****************************************************************************/

int  rfc2231_encode_string(char **);
void rfc2231_decode_parameters(PARAMETER **);

#endif /* MUTT_LIB_MIME_MIME_H */
